/*
 * Cay S. Horstmann & Gary Cornell, Core Java
 * Published By Sun Microsystems Press/Prentice-Hall
 * Copyright (C) 1997 Sun Microsystems Inc.
 * All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this 
 * software and its documentation for NON-COMMERCIAL purposes
 * and without fee is hereby granted provided that this 
 * copyright notice appears in all copies. 
 * 
 * THE AUTHORS AND PUBLISHER MAKE NO REPRESENTATIONS OR 
 * WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHORS
 * AND PUBLISHER SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED 
 * BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING 
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
 
/**
 * @version 1.10 27 Apr 1997
 * @author Cay Horstmann
 */

import java.awt.*;
import corejava.*;

public class CircleLayoutTest extends CloseableFrame
{  public CircleLayoutTest()
   {  setLayout(new CircleLayout());
      add(new Button("Yes"));
      add(new Button("No"));
      add(new Button("Ok"));
      add(new Button("Cancel"));
      add(new Button("Abort"));
      add(new Button("Retry"));
      add(new Button("Ignore"));
   }

   public static void main(String args[])
   {  Frame f = new CircleLayoutTest();
      f.show();
   }
   
}

class CircleLayout implements LayoutManager 
{  public void addLayoutComponent(String name, 
      Component comp) 
   {}

   public void removeLayoutComponent(Component comp) 
   {}

   public void setSizes(Container parent)
   {  if (sizesSet) return;
      int n = parent.getComponentCount();
   
      preferredWidth = 0;
      preferredHeight = 0;
      minWidth = 0;
      minHeight = 0;
      maxComponentWidth = 0;
      maxComponentHeight = 0;

      for (int i = 0; i < n; i++)
      {  Component c = parent.getComponent(i);
         if (c.isVisible()) {
         Dimension d = c.getPreferredSize();
         maxComponentWidth = Math.max(maxComponentWidth, 
            d.width);
         maxComponentHeight = Math.max(maxComponentWidth, 
            d.height);
         preferredHeight += d.height;
         }
      }
      preferredHeight += maxComponentHeight;
      preferredWidth = 2 * maxComponentWidth;
      minHeight = preferredHeight;       
      minWidth = preferredWidth;
      sizesSet = true;
   }

   public Dimension preferredLayoutSize(Container parent) 
   {  Dimension dim = new Dimension(0, 0);
      setSizes(parent);
      Insets insets = parent.getInsets();
      dim.width = preferredWidth + insets.left 
         + insets.right;
      dim.height = preferredHeight + insets.top 
         + insets.bottom;
      return dim;
   }

   public Dimension minimumLayoutSize(Container parent) 
   {  Dimension dim = new Dimension(0, 0);
      setSizes(parent);
      Insets insets = parent.getInsets();
      dim.width = minWidth + insets.left + insets.right;
      dim.height = minHeight + insets.top + insets.bottom;
      return dim;
   }

   public void layoutContainer(Container parent) 
   {  Insets insets = parent.getInsets();
      int containerWidth = parent.getSize().width 
         - insets.left - insets.right;
      int containerHeight = parent.getSize().height 
         - insets.top - insets.bottom;
      int xradius = (containerWidth - maxComponentWidth)
         / 2;
      int yradius = (containerHeight - maxComponentHeight)
         / 2;
        
      setSizes(parent);
      int xcenter = insets.left + containerWidth / 2;
      int ycenter = insets.top + containerHeight / 2;

      int n = parent.getComponentCount();
      for (int i = 0 ; i < n ; i++) 
      {  Component c = parent.getComponent(i);
         if (c.isVisible()) 
         {  Dimension d = c.getPreferredSize();
            double angle = 2 * Math.PI * i / n;
            int x = xcenter 
               + (int)(Math.cos(angle) * xradius);
            int y = ycenter 
               + (int)(Math.sin(angle) * yradius);
          
            c.setBounds(x - d.width / 2, y - d.width / 2, 
               d.width, d.height);
         }
      }

   }
   
   private int minWidth = 0;
   private int minHeight = 0;
   private int preferredWidth = 0, preferredHeight = 0;
   private boolean sizesSet = false;
   private int maxComponentWidth = 0;
   private int maxComponentHeight = 0;
}



